@@ -1,5 +1,5 @@
 # IMAP folder support
-# Copyright (C) 2002 John Goerzen
+# Copyright (C) 2002-2004 John Goerzen
 # <jgoerzen@complete.org>
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 from Base import BaseFolder
 from offlineimap import imaputil, imaplib
 from offlineimap.ui import UIBase
+from offlineimap.version import versionstr
 import rfc822, time, string, random, binascii, re
 from StringIO import StringIO
 from copy import copy
@@ -103,10 +104,15 @@ class IMAPFolder(BaseFolder):
         return self.messagelist
 
     def getmessage(self, uid):
+        ui = UIBase.getglobalui()
         imapobj = self.imapserver.acquireconnection()
         try:
             imapobj.select(self.getfullname(), readonly = 1)
-            return imapobj.uid('fetch', '%d' % uid, '(BODY.PEEK[])')[1][0][1].replace("\r\n", "\n")
+            initialresult = imapobj.uid('fetch', '%d' % uid, '(BODY.PEEK[])')
+            ui.debug('imap', 'Returned object from fetching %d: %s' % \
+                     (uid, str(initialresult)))
+            return initialresult[1][0][1].replace("\r\n", "\n")
+                
         finally:
             self.imapserver.releaseconnection(imapobj)
     
@@ -119,6 +125,7 @@ class IMAPFolder(BaseFolder):
         headername += binascii.hexlify(self.getname())
         headervalue= '%d-' % long(time.time())
         headervalue += str(self.randomgenerator.random()).replace('.', '')
+        headervalue += '-v' + versionstr
         return (headername, headervalue)
 
     def savemessage_addheader(self, content, headername, headervalue):
@@ -126,26 +133,24 @@ class IMAPFolder(BaseFolder):
         ui.debug('imap',
                  'savemessage_addheader: called to add %s: %s' % (headername,
                                                                   headervalue))
-        newline = "%s: %s" % (headername, headervalue)
-        ui.debug('imap', 'savemessage_addheader: newline = ' + repr(newline))
-
         insertionpoint = content.find("\r\n")
         ui.debug('imap', 'savemessage_addheader: insertionpoint = %d' % insertionpoint)
-
-        if insertionpoint == -1:
-            return newline + "\r\n" + content
-        elif insertionpoint == 0:
-            return newline + content
+        leader = content[0:insertionpoint]
+        ui.debug('imap', 'savemessage_addheader: leader = %s' % repr(leader))
+        if insertionpoint == 0 or insertionpoint == -1:
+            newline = ''
+            insertionpoint = 0
         else:
-            leader = content[0:insertionpoint]
-            ui.debug('imap', 'savemessage_addheader: leader = %s' % repr(leader))
-            trailer = content[insertionpoint:]
-            ui.debug('imap', 'savemessage_addheader: trailer = ' + repr(trailer))
-            return leader + "\r\n" + newline + trailer
+            newline = "\r\n"
+        newline += "%s: %s" % (headername, headervalue)
+        ui.debug('imap', 'savemessage_addheader: newline = ' + repr(newline))
+        trailer = content[insertionpoint:]
+        ui.debug('imap', 'savemessage_addheader: trailer = ' + repr(trailer))
+        return leader + newline + trailer
 
     def savemessage_searchforheader(self, imapobj, headername, headervalue):
         if imapobj.untagged_responses.has_key('APPENDUID'):
-            return long(imapobj.untagged_responses['APPENDUID'][0].split(' ')[1])
+            return long(imapobj.untagged_responses['APPENDUID'][-1].split(' ')[1])
 
         ui = UIBase.getglobalui()
         ui.debug('imap', 'savemessage_searchforheader called for %s: %s' % \
@@ -277,6 +282,12 @@ class IMAPFolder(BaseFolder):
         self.processmessagesflags('-', uidlist, flags)
 
     def processmessagesflags(self, operation, uidlist, flags):
+        if len(uidlist) > 101:
+            # Hack for those IMAP ervers with a limited line length
+            self.processmessagesflags(operation, uidlist[:100], flags)
+            self.processmessagesflags(operation, uidlist[100:], flags)
+            return
+        
         imapobj = self.imapserver.acquireconnection()
         try:
             try:
